@@ -1,4 +1,4 @@
-# Copyright (c) 2019, Riverbank Computing Limited
+# Copyright (c) 2020, Riverbank Computing Limited
 # All rights reserved.
 #
 # This copy of SIP is licensed for use under the terms of the SIP License
@@ -63,6 +63,10 @@ class Project(AbstractProject, Configurable):
         # Set if an __init__.py should be installed.
         Option('dunder_init', option_type=bool, default=False),
 
+        # The minimum GLIBC version required by the project.  This is used to
+        # determine the correct platform tag to use for Linux wheels.
+        Option('minimum_glibc_version'),
+
         # Set if building for a debug version of Python.
         Option('py_debug', option_type=bool),
 
@@ -81,7 +85,7 @@ class Project(AbstractProject, Configurable):
         # The name of the directory containing the .sip files.  If the sip
         # module is shared then each set of bindings is in its own
         # sub-directory.
-        Option('sip_files_dir', default=os.getcwd()),
+        Option('sip_files_dir', default='.'),
 
         # The list of files and directories, specified as glob patterns
         # relative to the project directory, that should be excluded from an
@@ -102,6 +106,8 @@ class Project(AbstractProject, Configurable):
         Option('name', help="the name used in sdist and wheel file names",
                 metavar="NAME", tools=['sdist', 'wheel']),
         Option('build_dir', help="the build directory", metavar="DIR"),
+        Option('build_tag', help="the build tag to be used in the wheel name",
+                metavar="TAG", tools=['wheel']),
         Option('target_dir', default=get_python_lib(plat_specific=1),
                 help="the target installation directory", metavar="DIR",
                 tools=['build', 'install']),
@@ -308,6 +314,21 @@ class Project(AbstractProject, Configurable):
 
             print(message, flush=True)
 
+    def project_path(self, path, relative_to=None):
+        """ Return a normalised version of a path.  A relative path is assumed
+        to be relate to the project directory or some other provided directory.
+        """
+
+        path = os.path.normpath(path)
+
+        if os.path.isabs(path):
+            return path
+
+        if relative_to is None:
+            relative_to = self.root_dir
+
+        return os.path.normpath(os.path.join(relative_to, path))
+
     def read_command_pipe(self, args, *, and_stderr=False, fatal=True):
         """ A generator for each line of a pipe from a command's stdout. """
 
@@ -351,6 +372,10 @@ class Project(AbstractProject, Configurable):
         # parts of the configuration.
         if tool != 'pep517':
             self._configure_from_command_line(tool, tool_description)
+        else:
+            # Until pip improves it's error reporting we give the user all the
+            # help we can.
+            self.verbose = True
 
         # Now that any help has been given we can report a missing
         # pyproject.toml file.
@@ -414,11 +439,34 @@ class Project(AbstractProject, Configurable):
     def verify_configuration(self, tool):
         """ Verify that the configuration is complete and consistent. """
 
+        # Make sure any build tag is valid.
+        if self.build_tag and not self.build_tag[0].isdigit():
+            raise PyProjectOptionException('build-tag',
+                    "'{0}' must begin with a digit".format(self.build_tag),
+                    section_name='tool.sip.project')
+
+        # Make sure any minimum GLIBC version is valid and convert it to a
+        # 2-tuple.
+        if self.minimum_glibc_version:
+            parts = self.minimum_glibc_version.split('.')
+
+            try:
+                if len(parts) != 2:
+                    raise ValueError()
+
+                self.minimum_glibc_version = (int(parts[0]), int(parts[1]))
+            except ValueError:
+                raise PyProjectOptionException('minimum-glibc-version',
+                        "'{0}' is an invalid GLIBC version number".format(
+                                self.minimum_glibc_version),
+                        section_name='tool.sip.project')
+        else:
+            self.minimum_glibc_version = (2, 5)
+
         # Make sure relevent paths are absolute and use native separators.
-        self.sip_files_dir = self.sip_files_dir.replace('/', os.sep)
-        if not os.path.isabs(self.sip_files_dir):
-            self.sip_files_dir = os.path.join(self.root_dir,
-                    self.sip_files_dir)
+        self.sip_files_dir = self.project_path(self.sip_files_dir)
+        self.sip_include_dirs = [self.project_path(d)
+                for d in self.sip_include_dirs]
 
         # Make sure we support the targeted version of Python.
         py_version = (self.py_major_version, self.py_minor_version)
